
/******************************************************************************
* MODULE     : keyw_define.cc
* DESCRIPTION: Definition of interactive constants
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <evaluator.hh>
#include <alias.hh>
namespace mmx {

void
evaluator_rep::kw_define_1 () {
  // INPUT: (define_1, cont, ...)
  generic decl= car (trace);
  generic val = decl[2];
  generic var = decl[1][1];
  generic type= decl[1][2];
  if (!is_symbol (var)) {
    error (var, "Not a symbol");
    return;
  }
  cur= type;
  rem= cons (KW_DEFINE_2, var, val, rem);
  // OUTPUT: (type, define_2, var, val, cont, ...)
}

void
evaluator_rep::kw_define_2 () {
  // INPUT: (define_2, type, var, val, cont, ...)
  value type= as_value (car (rem));
  if (type->type != ID_CLASS) {
    error (car (trace)[1][2], "Not a type");
    return;
  }
  cur= list (MMX_TRANSTYPE, caddr (rem), type);
  rem= cons (KW_DEFINE_3, cadr (rem), type->val, cdddr (rem));
  // OUTPUT: (body_decl, define_3, var, type, cont, ...)
}

void
evaluator_rep::kw_define_3 () {
  // INPUT: (define_3, val, var, type, cont, ...)
  generic decl= car (trace);
  generic val = as_value (car (rem)) -> val;
  generic var = cadr (rem);
  generic type= caddr (rem);

  if (is_func (decl, MMX_DEFINEALIAS, 2)) {
    list l (val);
    val = alias (car (l));
    type= list (ID_ALIAS, type);
  }

  declare (as_symbol (var), value (val, type));
  cur= cadddr (rem);
  rem= cons (VAL_NONE, cddddr (rem));
  // OUTPUT: (cont, void, ...)
  trace= cdr (trace);
}

} // namespace mmx
