
/******************************************************************************
* MODULE     : compound_types.cc
* DESCRIPTION: Compound types for the Mathemagix interpreter
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <extern.hh>
#include <single.hh>
#include <printer.hh>
#include <alias.hh>
#include <table.hh>
namespace mmx {

/******************************************************************************
* Aliases
******************************************************************************/

generic
class_alias (generic g) {
  return list (ID_ALIAS, g[1]);
}

generic
alias_assign (generic g) {
  alias a= as_alias (g[1]);
  a->ref= g[2];
  return a->ref;
}

/******************************************************************************
* Generators
******************************************************************************/

generic
class_generator (generic g) {
  return list (ID_GENERATOR, g[1]);
}

/******************************************************************************
* Lists
******************************************************************************/

generic
class_list (generic g) {
  return list (ID_LIST, g[1]);
}

generic
list_list (generic g) {
  return cdr (g);
}

generic
list_cons (generic g) {
  return cons (g[1], as_list (g[2]));
}

generic
list_car (generic g) {
  return car (as_list (g[1]));
}

generic
list_cdr (generic g) {
  return cdr (as_list (g[1]));
}

generic
list_write_car (generic g) {
  list l= as_list (as_alias (g[1]) -> ref);
  return alias (car (l));
}

generic
list_write_cdr (generic g) {
  list l= as_list (as_alias (g[1]) -> ref);
  return alias (*((generic*) ((void*) (&cdr (l)))));
}

generic
list_concat (generic g) {
  return as_list (g[1]) * as_list (g[2]);
}

generic
list_number (generic g) {
  return single (N (as_list (g[1])));
}

generic
list_nil (generic g) {
  return single (nil (as_list (g[1])));
}

generic
list_reverse (generic g) {
  return reverse (as_list (g[1]));
}

generic
list_access (generic g) {
  return as_list (g[1]) [as_int (g[2])];
}

generic
list_write_access (generic g) {
  list l= as_list (as_alias (g[1]) -> ref);
  return alias (l [as_int (g[2])]);
}

generic
list_output (generic g) {
  printer mmx_out= as_printer (g[1]);
  ostream& out (mmx_out->out);
  out << as_list (g[2]);
  return mmx_out;
}

/******************************************************************************
* Tables
******************************************************************************/

generic
class_table (generic g) {
  return list (ID_TABLE, g[1], g[2]);
}

generic
table_table (generic g) {
  return table (2, g[1]);
}

generic
table_read_access (generic g) {
  table t= as_table (as_alias (g[1]) -> ref);
  return t(g[2]);
}

generic
table_write_access (generic g) {
  table t= as_table (as_alias (g[1]) -> ref);
  return alias (t(g[2]));
}

generic
table_access (generic g) {
  return as_table (g[1]) [g[2]];
}

generic
table_number (generic g) {
  return single (N (as_table (g[1])));
}

generic
table_equal (generic g) {
  return single (as_table (g[1]) == as_table (g[2]));
}

generic
table_unequal (generic g) {
  return single (as_table (g[1]) != as_table (g[2]));
}

generic
table_output (generic g) {
  printer mmx_out= as_printer (g[1]);
  ostream& out (mmx_out->out);
  out << as_table (g[2]);
  return mmx_out;
}

/******************************************************************************
* Interface
******************************************************************************/

void
initialize_compound_types () {
  symbol T ("T");
  symbol U ("U");
  generic ID_TUPLET= list (ID_TUPLE, T);
  generic ID_CLASS_CLASS= function_type (ID_CLASS, ID_CLASS);
  generic ID_CLASS_CLASS_CLASS= function_type (ID_CLASS, ID_CLASS, ID_CLASS);

  // aliases
  generic ID_ALIAST= list (ID_ALIAS, T);
  generic ID_ALIASU= list (ID_ALIAS, U);
  generic ID_ALIAST_T_T= function_type (ID_ALIAST, T, T);
  generic ID_QT_ALIAST_T_T= template_type (T, ID_ALIAST_T_T);
  declare (MMX_ALIAS, class_alias, ID_CLASS_CLASS);
  declare (MMX_ASSIGN, alias_assign, ID_QT_ALIAST_T_T);

  // generators
  declare (MMX_GENERATOR, class_generator, ID_CLASS_CLASS);

  // lists
  generic ID_LISTT= list (ID_LIST, T);
  generic ID_ALIAS_LISTT= list (ID_ALIAS, ID_LISTT);
  generic ID_TUPLET_LISTT= function_type (ID_TUPLET, ID_LISTT);
  generic ID_QT_TUPLET_LISTT= template_type (T, ID_TUPLET_LISTT);
  generic ID_T_LISTT_LISTT= function_type (T, ID_LISTT, ID_LISTT);
  generic ID_QT_T_LISTT_LISTT= template_type (T, ID_T_LISTT_LISTT);
  generic ID_LISTT_T= function_type (ID_LISTT, T);
  generic ID_QT_LISTT_T= template_type (T, ID_LISTT_T);
  generic ID_LISTT_LISTT= function_type (ID_LISTT, ID_LISTT);
  generic ID_QT_LISTT_LISTT= template_type (T, ID_LISTT_LISTT);
  generic ID_ALIAS_LISTT_ALIAS_T= function_type (ID_ALIAS_LISTT, ID_ALIAST);
  generic ID_QT_ALIAS_LISTT_ALIAS_T= template_type (T, ID_ALIAS_LISTT_ALIAS_T);
  generic ID_ALIAS_LISTT_ALIAS_LISTT=
    function_type (ID_ALIAS_LISTT, ID_ALIAS_LISTT);
  generic ID_QT_ALIAS_LISTT_ALIAS_LISTT=
    template_type (T, ID_ALIAS_LISTT_ALIAS_LISTT);
  generic ID_LISTT_LISTT_LISTT= function_type (ID_LISTT, ID_LISTT, ID_LISTT);
  generic ID_QT_LISTT_LISTT_LISTT= template_type (T, ID_LISTT_LISTT_LISTT);
  generic ID_LISTT_INT= function_type (ID_LISTT, ID_INT);
  generic ID_QT_LISTT_INT= template_type (T, ID_LISTT_INT);
  generic ID_LISTT_BOOL= function_type (ID_LISTT, ID_BOOL);
  generic ID_QT_LISTT_BOOL= template_type (T, ID_LISTT_BOOL);
  generic ID_LISTT_INT_T= function_type (ID_LISTT, ID_INT, T);
  generic ID_QT_LISTT_INT_T= template_type (T, ID_LISTT_INT_T);
  generic ID_ALIAS_LISTT_INT_ALIAS_T=
    function_type (ID_ALIAS_LISTT, ID_INT, ID_ALIAST);
  generic ID_QT_ALIAS_LISTT_INT_ALIAS_T=
    template_type (T, ID_ALIAS_LISTT_INT_ALIAS_T);
  generic ID_OSTREAM_LISTT_OSTREAM=
    function_type (ID_OSTREAM, ID_LISTT, ID_OSTREAM);
  generic ID_QTU_OSTREAM_LISTT_OSTREAM=
    template_type (T, ID_OSTREAM_LISTT_OSTREAM);

  declare (MMX_LIST, class_list, ID_CLASS_CLASS);
  declare (MMX_SQTUPLE, list_list, ID_QT_TUPLET_LISTT);
  declare ("cons", list_cons, ID_QT_T_LISTT_LISTT);
  declare (MMX_BACKPRIME, list_car, ID_QT_LISTT_T);
  declare (MMX_PRIME, list_cdr, ID_QT_LISTT_LISTT);
  declare (MMX_BACKPRIME, list_write_car, ID_QT_ALIAS_LISTT_ALIAS_T);
  declare (MMX_PRIME, list_write_cdr, ID_QT_ALIAS_LISTT_ALIAS_LISTT);
  declare (MMX_TIMES, list_concat, ID_QT_LISTT_LISTT_LISTT);
  declare (MMX_NUMBER, list_number, ID_QT_LISTT_INT);
  declare ("nil", list_nil, ID_QT_LISTT_BOOL);
  declare ("reverse", list_reverse, ID_QT_LISTT_LISTT);
  declare (MMX_ACCESS, list_access, ID_QT_LISTT_INT_T);
  declare (MMX_ACCESS, list_write_access, ID_QT_ALIAS_LISTT_INT_ALIAS_T);
  declare (MMX_LESSLESS, list_output, ID_QTU_OSTREAM_LISTT_OSTREAM);

  // tables
  generic ID_TABLETU= list (ID_TABLE, T, U);
  generic ID_QT_TABLETU= template_type (T, ID_TABLETU);
  generic ID_U_QT_TABLETU= function_type (U, ID_QT_TABLETU);
  generic ID_QU_U_QT_TABLETU= template_type (U, ID_U_QT_TABLETU);
  generic ID_TABLETU_T_U= function_type (ID_TABLETU, T, U);
  generic ID_QTU_TABLETU_T_U= template_type (T, U, ID_TABLETU_T_U);
  generic ID_ALIAS_TABLETU_T_ALIAS_U=
    function_type (list (ID_ALIAS, ID_TABLETU), T, ID_ALIASU);
  generic ID_QTU_ALIAS_TABLETU_T_ALIAS_U=
    template_type (T, U, ID_ALIAS_TABLETU_T_ALIAS_U);
  generic ID_TABLETU_INT= function_type (ID_TABLETU, ID_INT);
  generic ID_QTU_TABLETU_INT= template_type (T, U, ID_TABLETU_INT);
  generic ID_TABLETU_TABLETU_BOOL=
    function_type (ID_TABLETU, ID_TABLETU, ID_BOOL);
  generic ID_QTU_TABLETU_TABLETU_BOOL=
    template_type (T, U, ID_TABLETU_TABLETU_BOOL);
  generic ID_OSTREAM_TABLETU_OSTREAM=
    function_type (ID_OSTREAM, ID_TABLETU, ID_OSTREAM);
  generic ID_QTU_OSTREAM_TABLETU_OSTREAM=
    template_type (T, U, ID_OSTREAM_TABLETU_OSTREAM);

  declare (MMX_TABLE, class_table, ID_CLASS_CLASS_CLASS);
  declare ("table", table_table, ID_QU_U_QT_TABLETU);
  declare (MMX_ACCESS, table_access, ID_QTU_TABLETU_T_U);
  declare (MMX_ACCESS, table_write_access, ID_QTU_ALIAS_TABLETU_T_ALIAS_U);
  declare (MMX_NUMBER, table_number, ID_QTU_TABLETU_INT);
  declare (MMX_EQUAL, table_equal, ID_QTU_TABLETU_TABLETU_BOOL);
  declare (MMX_UNEQUAL, table_unequal, ID_QTU_TABLETU_TABLETU_BOOL);
  declare (MMX_LESSLESS, table_output, ID_QTU_OSTREAM_TABLETU_OSTREAM);
}

} // namespace mmx
