
#ifndef MMX_SYNAPS_H
#define MMX_SYNAPS_H

#include <extern.hh>
#include <Synaps/mmx_extern.H>
#include "synaps/linalg.H"
#include "synaps/upol.H"
#include "synaps/arithm/ZZ.H" 
#include "synaps/solve/Aberth.H" 
#include "synaps/mpol.H"

//--------------------------------------------------------------------
symbol MM_ZZ ("ZZ");
generic as_zz (generic g) 
{
  ZZ x(as_charp(as_mm_string(g[1])));
  return mm_obj<ZZ>(x);
}
generic zz_mult (generic g) 
{
  return mm_obj<ZZ>(as_mm_obj<ZZ>(g[1])->data*as_mm_obj<ZZ>(g[2])->data);
}

generic zz_plus (generic g) 
{
  return mm_obj<ZZ>(as_mm_obj<ZZ>(g[1])->data+as_mm_obj<ZZ>(g[2])->data);
}
//--------------------------------------------------------------------
symbol MM_UPOL ("UPol");
generic as_upol (generic g) 
{
  typedef UPolDse<double> C;
  C x(as_charp(as_mm_string(g[1])));
  return mm_obj<C>(x);
}
generic upol_mult (generic g) 
{
  typedef UPolDse<double> C;
  return mm_obj<C>(as_mm_obj<C>(g[1])->data*as_mm_obj<C>(g[2])->data);
}
generic upol_plus (generic g) 
{
  typedef UPolDse<double> C;
  return mm_obj<C>(as_mm_obj<C>(g[1])->data+as_mm_obj<C>(g[2])->data);
}
generic upol_solve (generic g) 
{
  typedef UPolDse<double> C;
  cout <<as_mm_obj<C>(g[1])->data<<"\n";
  cout <<Solve(as_mm_obj<C>(g[1])->data, Aberth())<<"\n";
  return as_mm_obj<C>(g[1]);
}

//--------------------------------------------------------------------
symbol MM_MPOL ("MPol");
generic as_mpol (generic g) 
{
  typedef MPol<double> C;
  C x(as_charp(as_mm_string(g[1])));
  return mm_obj<C>(x);
}

//--------------------------------------------------------------------
symbol MM_VECTOR ("Vector");
generic as_vector (generic g) 
{
  typedef VectDse<double> C;
  C x(as_int(g[1]), as_charp(as_mm_string(g[2])));
  return mm_obj<C>(x);
}
//--------------------------------------------------------------------
symbol MM_MATRIX ("Matrix");
generic as_matrix (generic g) 
{
  typedef MatrDse<double> C;
  C x(as_int(g[1]), as_int(g[2]), as_charp(as_mm_string(g[3])));
  return mm_obj<C>(x);
}
/******************************************************************************
* Interface
******************************************************************************/

void
initialize_synaps () {
  // This should go into Gmp instead of Galaad
  generic MM_INTEGER_INTEGER_INTEGER=
    function_type (MM_INTEGER, MM_INTEGER, MM_INTEGER);

  declare ("integer", as_zz, function_type (MM_STRING, MM_INTEGER));
    // use lowercase for constructors and functions in general
  declare (MM_TIMES, zz_mult, MM_INTEGER_INTEGER_INTEGER);
  declare (MM_PLUS, zz_plus, MM_INTEGER_INTEGER_INTEGER);

  // Synaps specific; OK
  declare ("upol", as_upol, function_type (MM_STRING, MM_UPOL));
  declare (MM_TIMES, upol_mult, function_type (MM_UPOL, MM_UPOL, MM_UPOL));
  declare (MM_PLUS,  upol_plus, function_type (MM_UPOL, MM_UPOL, MM_UPOL));
  declare ("solve", upol_solve, function_type (MM_UPOL, MM_UPOL));
  declare ("mpol",     as_mpol, function_type (MM_STRING, MM_MPOL));
  declare ("matrix", as_matrix, function_type (MM_INT, MM_INT,MM_STRING, MM_MATRIX));
  declare ("vector", as_vector, function_type (MM_INT, MM_STRING, MM_VECTOR));
}

#endif //MMX_SYNAPS_H
