
/******************************************************************************
* MODULE     : value.cc
* DESCRIPTION: Mathemagix values with a unique type
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <function.hh>
#include <symbol.hh>
#include <value.hh>
namespace mmx {

/******************************************************************************
* Routines for the generic type
******************************************************************************/

int
value_rep::Type () {
  return VALUE_TYPE;
}

bool
value_rep::Equal (generic g) {
  if (gentype (g) != VALUE_TYPE) return false;
  return value (this) == as_value (g);
}

generic
value_rep::Copy () {
  return copy (value (this));
}

void
value_rep::Print (ostream& out) {
  out << value (this);
}

/******************************************************************************
* Routines on values
******************************************************************************/

ostream&
operator << (ostream& out, value val) {
  out << "value<" << val->val << "," << val->type << ">";
  return out;
}

value
copy (value val) {
  return value (val->val, val->type);
}

/******************************************************************************
* Manipulation of lists of values and types
******************************************************************************/

list
get_values (list l) {
  if (nil (l)) return l;
  return cons (as_value (car (l)) -> val, get_values (cdr (l)));
}

list
get_types (list l) {
  if (nil (l)) return l;
  return cons (as_value (car (l)) -> type, get_types (cdr (l)));
}

list
flatten_vars (list l) {
  if (nil (l)) return l;
  if (is_func (car (l), ":" /* FIXME: MMX_TYPE */, 2))
    return cons (car (l) [1], flatten_vars (cdr (l)));
  return cons (car (l), flatten_vars (cdr (l)));
}

list
flatten_types (list l) {
  if (nil (l)) return l;
  if (is_func (car (l), ":" /* FIXME: MMX_TYPE */, 2))
    return cons (car (l) [2], flatten_types (cdr (l)));
  return cons (car (l), flatten_types (cdr (l)));
}

} // namespace mmx
